@@ -1,27 +1,19 @@
-#---------------------------------------------#
-#
-# Mailer will queue up emails, Try to send them
-# and keep track of if they are sent or not.
-# Should be executed with a cron job.
-#
-#---------------------------------------------#
-from django.utils.encoding import python_2_unicode_compatible
-from django.utils.translation import ugettext_lazy as _
 import datetime
 import logging
 import os
 
-logger = logging.getLogger(__name__)
 
-from django.utils import timezone
-from django.db import models
-from django.core.mail import EmailMultiAlternatives
-from django.db.models.signals import post_save
-from django.dispatch import receiver
 from django.conf import settings
+from django.core.files.base import ContentFile
+from django.core.mail import EmailMultiAlternatives
+from django.db import models
+from django.utils.translation import gettext_lazy as _
+from django.utils import timezone
 
 from . import defaults
-from .utils import get_storage
+from .utils import get_storage, upload_to
+
+logger = logging.getLogger(__name__)
 
 
 class MailerMessageManager(models.Manager):
@@ -44,17 +36,21 @@ class MailerMessageManager(models.Manager):
         self.filter(sent=True, last_attempt__lte=delete_before).delete()
 
 
-@python_2_unicode_compatible
 class MailerMessage(models.Model):
+    created = models.DateTimeField(_('Created'), auto_now_add=True, auto_now=False,
+                                   editable=False, null=True)
     subject = models.CharField(_('Subject'), max_length=250, blank=True)
     to_address = models.TextField(_('To'))
+    cc_address = models.TextField(_('CC'), blank=True)
     bcc_address = models.TextField(_('BCC'), blank=True)
     from_address = models.EmailField(_('From'), max_length=250)
+    reply_to = models.TextField(_('Reply to'), max_length=250, blank=True, null=True)
     content = models.TextField(_('Content'), blank=True)
     html_content = models.TextField(_('HTML Content'), blank=True)
     app = models.CharField(_('App'), max_length=250, blank=True)
-    sent = models.BooleanField(_('Sent'), default=False, editable=False)
-    last_attempt = models.DateTimeField(_('Last attempt'), auto_now=False, auto_now_add=False, blank=True, null=True, editable=False)
+    sent = models.BooleanField(_('Sent'), default=False, editable=True)
+    last_attempt = models.DateTimeField(_('Last attempt'), auto_now=False, auto_now_add=False,
+                                        blank=True, null=True, editable=False)
 
     objects = MailerMessageManager()
 
@@ -72,7 +68,18 @@ class MailerMessage(models.Model):
         if self.pk is None:
             self._save_without_sending()
 
-        Attachment.objects.create(email=self, file_attachment=attachment)
+        original_filename = attachment.file.name.split(os.sep)[-1]
+        file_content = ContentFile(attachment.read())
+
+        new_attachment = Attachment()
+        new_attachment.file_attachment.save(original_filename, file_content, save=False)
+        new_attachment.email = self
+        new_attachment.original_filename = original_filename
+        try:
+            new_attachment.save()
+        except Exception as e:
+            logger.error(e)
+            new_attachment.file_attachment.delete()
 
     def _save_without_sending(self, *args, **kwargs):
         """
@@ -99,17 +106,28 @@ class MailerMessage(models.Model):
 
             subject, from_email = self.subject, self.from_address
             text_content = self.content
+
             msg = EmailMultiAlternatives(subject, text_content, from_email)
+
+            if self.reply_to:
+                msg.reply_to = [email.strip() for email in self.reply_to.split(',')
+                                if email.strip()]
+
             if self.html_content:
                 html_content = self.html_content
                 msg.attach_alternative(html_content, "text/html")
 
             msg.to = [email.strip() for email in self.to_address.split(',') if email.strip()]
+            msg.cc = [email.strip() for email in self.cc_address.split(',') if email.strip()]
             msg.bcc = [email.strip() for email in self.bcc_address.split(',') if email.strip()]
 
             # Add any additional attachments
             for attachment in self.attachment_set.all():
-                msg.attach_file(os.path.join(settings.MEDIA_ROOT, attachment.file_attachment.name))
+                path = attachment.file_attachment.path
+                if os.path.isfile(path):
+                    with open(path, 'rb') as f:
+                        content = f.read()
+                    msg.attach(attachment.original_filename, content, None)
             try:
                 msg.send()
                 self.sent = True
@@ -119,25 +137,15 @@ class MailerMessage(models.Model):
             self.save()
 
 
-@python_2_unicode_compatible
 class Attachment(models.Model):
-    file_attachment = models.FileField(storage=get_storage(), upload_to='mail-queue/attachments', blank=True, null=True)
-    email = models.ForeignKey(MailerMessage, blank=True, null=True)
+    file_attachment = models.FileField(storage=get_storage(), upload_to=upload_to,
+                                       blank=True, null=True)
+    original_filename = models.CharField(default=None, max_length=250, blank=False)
+    email = models.ForeignKey(MailerMessage, on_delete=models.CASCADE, blank=True, null=True)
 
     class Meta:
         verbose_name = _('Attachment')
         verbose_name_plural = _('Attachments')
 
     def __str__(self):
-        return self.file_attachment.name
-
-@receiver(post_save, sender=MailerMessage)
-def send_post_save(sender, instance, signal, *args, **kwargs):
-    if getattr(instance, "do_not_send", False):
-        instance.do_not_send = False
-        return
-
-    if not getattr(settings, 'MAILQUEUE_QUEUE_UP', defaults.MAILQUEUE_QUEUE_UP):
-        # If mail queue up is set, wait for the cron or management command
-        # to send any email.
-        instance.send_mail()
+        return str(self.original_filename)
